package net.sqlcipher.database;

import android.content.ContentValues;
import android.content.Context;
import android.content.res.AssetManager;
import android.os.Debug;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Log;
import android.util.Pair;
import com.google.common.collect.Maps;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.lang.ref.WeakReference;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.Set;
import java.util.WeakHashMap;
import java.util.concurrent.locks.ReentrantLock;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipInputStream;
import net.sqlcipher.CrossProcessCursorWrapper;
import net.sqlcipher.Cursor;
import net.sqlcipher.DatabaseUtils;
import net.sqlcipher.SQLException;

public class SQLiteDatabase extends SQLiteClosable
{
  private static final String COMMIT_SQL = "COMMIT;";
  public static final int CONFLICT_ABORT = 2;
  public static final int CONFLICT_FAIL = 3;
  public static final int CONFLICT_IGNORE = 4;
  public static final int CONFLICT_NONE = 0;
  public static final int CONFLICT_REPLACE = 5;
  public static final int CONFLICT_ROLLBACK = 1;
  private static final String[] CONFLICT_VALUES = { "", " OR ROLLBACK ", " OR ABORT ", " OR FAIL ", " OR IGNORE ", " OR REPLACE " };
  public static final int CREATE_IF_NECESSARY = 268435456;
  private static final Pattern EMAIL_IN_DB_PATTERN = Pattern.compile("[\\w\\.\\-]+@[\\w\\.\\-]+");
  private static final int EVENT_DB_CORRUPT = 75004;
  private static final int EVENT_DB_OPERATION = 52000;
  static final String GET_LOCK_LOG_PREFIX = "GETLOCK:";
  private static final int LOCK_ACQUIRED_WARNING_THREAD_TIME_IN_MS = 100;
  private static final int LOCK_ACQUIRED_WARNING_TIME_IN_MS = 300;
  private static final int LOCK_ACQUIRED_WARNING_TIME_IN_MS_ALWAYS_PRINT = 2000;
  private static final int LOCK_WARNING_WINDOW_IN_MS = 20000;
  private static final String LOG_SLOW_QUERIES_PROPERTY = "db.log.slow_query_threshold";
  public static final int MAX_SQL_CACHE_SIZE = 250;
  private static final int MAX_WARNINGS_ON_CACHESIZE_CONDITION = 1;
  public static final int NO_LOCALIZED_COLLATORS = 16;
  public static final int OPEN_READONLY = 1;
  public static final int OPEN_READWRITE = 0;
  private static final int OPEN_READ_MASK = 1;
  private static final int QUERY_LOG_SQL_LENGTH = 64;
  private static final int SLEEP_AFTER_YIELD_QUANTUM = 1000;
  public static final int SQLITE_MAX_LIKE_PATTERN_LENGTH = 50000;
  private static final String TAG = "Database";
  private static int sQueryLogTimeInMillis = 0;
  private int mCacheFullWarnings;
  Map<String, SQLiteCompiledSql> mCompiledQueries = Maps.newHashMap();
  private CursorFactory mFactory;
  private int mFlags;
  private boolean mInnerTransactionIsSuccessful;
  private long mLastLockMessageTime = 0L;
  private String mLastSqlStatement = null;
  private final ReentrantLock mLock = new ReentrantLock(true);
  private long mLockAcquiredThreadTime = 0L;
  private long mLockAcquiredWallTime = 0L;
  private boolean mLockingEnabled = true;
  private int mMaxSqlCacheSize = 250;
  int mNativeHandle = 0;
  private int mNumCacheHits;
  private int mNumCacheMisses;
  private String mPath;
  private String mPathForLogs = null;
  private WeakHashMap<SQLiteClosable, Object> mPrograms;
  private final Random mRandom = new Random();
  private final int mSlowQueryThreshold;
  private Throwable mStackTrace = null;
  private final Map<String, SyncUpdateInfo> mSyncUpdateInfo = new HashMap();
  int mTempTableSequence = 0;
  private String mTimeClosed = null;
  private String mTimeOpened = null;
  private boolean mTransactionIsSuccessful;
  private SQLiteTransactionListener mTransactionListener;

  public SQLiteDatabase(String paramString1, String paramString2, CursorFactory paramCursorFactory, int paramInt)
  {
    this(paramString1, paramString2, paramCursorFactory, paramInt, null);
  }

  public SQLiteDatabase(String paramString1, String paramString2, CursorFactory paramCursorFactory, int paramInt, SQLiteDatabaseHook paramSQLiteDatabaseHook)
  {
    if (paramString1 == null)
      throw new IllegalArgumentException("path should not be null");
    this.mFlags = paramInt;
    this.mPath = paramString1;
    this.mSlowQueryThreshold = -1;
    this.mStackTrace = new DatabaseObjectNotClosedException().fillInStackTrace();
    this.mFactory = paramCursorFactory;
    dbopen(this.mPath, this.mFlags);
    if (paramSQLiteDatabaseHook != null)
      paramSQLiteDatabaseHook.preKey(this);
    native_key(paramString2.toCharArray());
    if (paramSQLiteDatabaseHook != null)
      paramSQLiteDatabaseHook.postKey(this);
    if (SQLiteDebug.DEBUG_SQL_CACHE)
      this.mTimeOpened = getTime();
    this.mPrograms = new WeakHashMap();
    try
    {
      setLocale(Locale.getDefault());
      return;
    }
    catch (RuntimeException localRuntimeException)
    {
      Log.e("Database", "Failed to setLocale() when constructing, closing the database", localRuntimeException);
      dbclose();
      if (SQLiteDebug.DEBUG_SQL_CACHE)
        this.mTimeClosed = getTime();
      throw localRuntimeException;
    }
  }

  private void checkLockHoldTime()
  {
    long l1 = SystemClock.elapsedRealtime();
    long l2 = l1 - this.mLockAcquiredWallTime;
    if ((l2 < 2000L) && (!Log.isLoggable("Database", 2)) && (l1 - this.mLastLockMessageTime < 20000L));
    int i;
    do
    {
      do
        return;
      while (l2 <= 300L);
      i = (int)((Debug.threadCpuTimeNanos() - this.mLockAcquiredThreadTime) / 1000000L);
    }
    while ((i <= 100) && (l2 <= 2000L));
    this.mLastLockMessageTime = l1;
    String str = "lock held on " + this.mPath + " for " + l2 + "ms. Thread time was " + i + "ms";
    if (SQLiteDebug.DEBUG_LOCK_TIME_TRACKING_STACK_TRACE)
    {
      Log.d("Database", str, new Exception());
      return;
    }
    Log.d("Database", str);
  }

  private void closeClosable()
  {
    deallocCachedSqlStatements();
    Iterator localIterator = this.mPrograms.entrySet().iterator();
    while (localIterator.hasNext())
    {
      SQLiteClosable localSQLiteClosable = (SQLiteClosable)((Map.Entry)localIterator.next()).getKey();
      if (localSQLiteClosable != null)
        localSQLiteClosable.onAllReferencesReleasedFromContainer();
    }
  }

  public static SQLiteDatabase create(CursorFactory paramCursorFactory, String paramString)
  {
    return openDatabase(":memory:", paramString, paramCursorFactory, 268435456);
  }

  private native void dbclose();

  private native void dbopen(String paramString, int paramInt);

  private void deallocCachedSqlStatements()
  {
    synchronized (this.mCompiledQueries)
    {
      Iterator localIterator = this.mCompiledQueries.values().iterator();
      if (localIterator.hasNext())
        ((SQLiteCompiledSql)localIterator.next()).releaseSqlStatement();
    }
    this.mCompiledQueries.clear();
  }

  private native void enableSqlProfiling(String paramString);

  private native void enableSqlTracing(String paramString);

  public static String findEditTable(String paramString)
  {
    if (!TextUtils.isEmpty(paramString))
    {
      int i = paramString.indexOf(' ');
      int j = paramString.indexOf(',');
      if ((i > 0) && ((i < j) || (j < 0)))
        paramString = paramString.substring(0, i);
      while ((j <= 0) || ((j >= i) && (i >= 0)))
        return paramString;
      return paramString.substring(0, j);
    }
    throw new IllegalStateException("Invalid tables");
  }

  private static ArrayList<Pair<String, String>> getAttachedDbs(SQLiteDatabase paramSQLiteDatabase)
  {
    if (!paramSQLiteDatabase.isOpen())
      return null;
    ArrayList localArrayList = new ArrayList();
    Cursor localCursor = paramSQLiteDatabase.rawQuery("pragma database_list;", null);
    while (localCursor.moveToNext())
      localArrayList.add(new Pair(localCursor.getString(1), localCursor.getString(2)));
    localCursor.close();
    return localArrayList;
  }

  static ArrayList<SQLiteDebug.DbStats> getDbStats()
  {
    ArrayList localArrayList1 = new ArrayList();
    Iterator localIterator = ActiveDatabases.getInstance().mActiveDatabases.iterator();
    while (localIterator.hasNext())
    {
      SQLiteDatabase localSQLiteDatabase = (SQLiteDatabase)((WeakReference)localIterator.next()).get();
      if ((localSQLiteDatabase != null) && (localSQLiteDatabase.isOpen()))
      {
        int i = localSQLiteDatabase.native_getDbLookaside();
        String str1 = localSQLiteDatabase.getPath();
        int j = str1.lastIndexOf("/");
        int k;
        label87: int m;
        label110: Pair localPair;
        long l;
        String str3;
        if (j != -1)
        {
          k = j + 1;
          String str2 = str1.substring(k);
          ArrayList localArrayList2 = getAttachedDbs(localSQLiteDatabase);
          if (localArrayList2 != null)
          {
            m = 0;
            if (m < localArrayList2.size())
            {
              localPair = (Pair)localArrayList2.get(m);
              l = getPragmaVal(localSQLiteDatabase, (String)localPair.first + ".page_count;");
              if (m != 0)
                break label214;
              str3 = str2;
            }
          }
        }
        else
        {
          label214: int n;
          do
          {
            if (l > 0L)
              localArrayList1.add(new SQLiteDebug.DbStats(str3, l, localSQLiteDatabase.getPageSize(), i));
            m++;
            break label110;
            break;
            k = 0;
            break label87;
            str3 = "  (attached) " + (String)localPair.first;
            n = ((String)localPair.second).trim().length();
            i = 0;
          }
          while (n <= 0);
          int i1 = ((String)localPair.second).lastIndexOf("/");
          StringBuilder localStringBuilder = new StringBuilder().append(str3).append(" : ");
          String str4 = (String)localPair.second;
          if (i1 != -1);
          for (int i2 = i1 + 1; ; i2 = 0)
          {
            str3 = str4.substring(i2);
            i = 0;
            break;
          }
        }
      }
    }
    return localArrayList1;
  }

  private String getPathForLogs()
  {
    if (this.mPathForLogs != null)
      return this.mPathForLogs;
    if (this.mPath == null)
      return null;
    if (this.mPath.indexOf('@') == -1);
    for (this.mPathForLogs = this.mPath; ; this.mPathForLogs = EMAIL_IN_DB_PATTERN.matcher(this.mPath).replaceAll("XX@YY"))
      return this.mPathForLogs;
  }

  // ERROR //
  private static long getPragmaVal(SQLiteDatabase paramSQLiteDatabase, String paramString)
  {
    // Byte code:
    //   0: aload_0
    //   1: invokevirtual 413	net/sqlcipher/database/SQLiteDatabase:isOpen	()Z
    //   4: ifne +9 -> 13
    //   7: lconst_0
    //   8: lstore 7
    //   10: lload 7
    //   12: lreturn
    //   13: aconst_null
    //   14: astore_2
    //   15: new 531	net/sqlcipher/database/SQLiteStatement
    //   18: dup
    //   19: aload_0
    //   20: new 294	java/lang/StringBuilder
    //   23: dup
    //   24: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   27: ldc_w 533
    //   30: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   33: aload_1
    //   34: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   37: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   40: invokespecial 536	net/sqlcipher/database/SQLiteStatement:<init>	(Lnet/sqlcipher/database/SQLiteDatabase;Ljava/lang/String;)V
    //   43: astore_3
    //   44: aload_3
    //   45: invokevirtual 539	net/sqlcipher/database/SQLiteStatement:simpleQueryForLong	()J
    //   48: lstore 5
    //   50: lload 5
    //   52: lstore 7
    //   54: aload_3
    //   55: ifnull -45 -> 10
    //   58: aload_3
    //   59: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   62: lload 7
    //   64: lreturn
    //   65: astore 4
    //   67: aload_2
    //   68: ifnull +7 -> 75
    //   71: aload_2
    //   72: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   75: aload 4
    //   77: athrow
    //   78: astore 4
    //   80: aload_3
    //   81: astore_2
    //   82: goto -15 -> 67
    //
    // Exception table:
    //   from	to	target	type
    //   15	44	65	finally
    //   44	50	78	finally
  }

  private String getTime()
  {
    return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS ").format(Long.valueOf(System.currentTimeMillis()));
  }

  private static void loadICUData(Context paramContext, File paramFile)
  {
    ZipInputStream localZipInputStream;
    FileOutputStream localFileOutputStream;
    try
    {
      File localFile1 = new File(paramFile, "icu");
      if (!localFile1.exists())
        localFile1.mkdirs();
      File localFile2 = new File(localFile1, "icudt46l.dat");
      if (!localFile2.exists())
      {
        localZipInputStream = new ZipInputStream(paramContext.getAssets().open("icudt46l.zip"));
        localZipInputStream.getNextEntry();
        localFileOutputStream = new FileOutputStream(localFile2);
        byte[] arrayOfByte = new byte[1024];
        while (true)
        {
          int i = localZipInputStream.read(arrayOfByte);
          if (i <= 0)
            break;
          localFileOutputStream.write(arrayOfByte, 0, i);
        }
      }
    }
    catch (Exception localException)
    {
      Log.e("Database", "Error copying icu data file", localException);
    }
    return;
    localZipInputStream.close();
    localFileOutputStream.flush();
    localFileOutputStream.close();
  }

  public static void loadLibs(Context paramContext)
  {
    loadLibs(paramContext, paramContext.getFilesDir());
  }

  public static void loadLibs(Context paramContext, File paramFile)
  {
    System.loadLibrary("stlport_shared");
    System.loadLibrary("sqlcipher_android");
    System.loadLibrary("database_sqlcipher");
    boolean bool = new File("/system/usr/icu/icudt46l.dat").exists();
    if (bool);
    for (String str = "/system/usr"; ; str = paramFile.getAbsolutePath())
    {
      setICURoot(str);
      if (!bool)
        loadICUData(paramContext, paramFile);
      return;
    }
  }

  private void lockForced()
  {
    this.mLock.lock();
    if ((SQLiteDebug.DEBUG_LOCK_TIME_TRACKING) && (this.mLock.getHoldCount() == 1))
    {
      this.mLockAcquiredWallTime = SystemClock.elapsedRealtime();
      this.mLockAcquiredThreadTime = Debug.threadCpuTimeNanos();
    }
  }

  // ERROR //
  private void markTableSyncable(String paramString1, String paramString2, String paramString3, String paramString4)
  {
    // Byte code:
    //   0: aload_0
    //   1: invokevirtual 665	net/sqlcipher/database/SQLiteDatabase:lock	()V
    //   4: aload_0
    //   5: new 294	java/lang/StringBuilder
    //   8: dup
    //   9: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   12: ldc_w 667
    //   15: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   18: aload_3
    //   19: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   22: ldc_w 669
    //   25: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   28: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   31: invokevirtual 672	net/sqlcipher/database/SQLiteDatabase:native_execSQL	(Ljava/lang/String;)V
    //   34: aload_0
    //   35: new 294	java/lang/StringBuilder
    //   38: dup
    //   39: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   42: ldc_w 674
    //   45: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   48: aload_2
    //   49: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   52: ldc_w 676
    //   55: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   58: aload_1
    //   59: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   62: ldc_w 669
    //   65: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   68: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   71: invokevirtual 672	net/sqlcipher/database/SQLiteDatabase:native_execSQL	(Ljava/lang/String;)V
    //   74: aload_0
    //   75: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   78: new 681	net/sqlcipher/database/SQLiteDatabase$SyncUpdateInfo
    //   81: dup
    //   82: aload_3
    //   83: aload 4
    //   85: aload_2
    //   86: invokespecial 684	net/sqlcipher/database/SQLiteDatabase$SyncUpdateInfo:<init>	(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
    //   89: astore 6
    //   91: aload_0
    //   92: getfield 190	net/sqlcipher/database/SQLiteDatabase:mSyncUpdateInfo	Ljava/util/Map;
    //   95: astore 7
    //   97: aload 7
    //   99: monitorenter
    //   100: aload_0
    //   101: getfield 190	net/sqlcipher/database/SQLiteDatabase:mSyncUpdateInfo	Ljava/util/Map;
    //   104: aload_1
    //   105: aload 6
    //   107: invokeinterface 688 3 0
    //   112: pop
    //   113: aload 7
    //   115: monitorexit
    //   116: return
    //   117: astore 5
    //   119: aload_0
    //   120: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   123: aload 5
    //   125: athrow
    //   126: astore 8
    //   128: aload 7
    //   130: monitorexit
    //   131: aload 8
    //   133: athrow
    //
    // Exception table:
    //   from	to	target	type
    //   4	74	117	finally
    //   100	116	126	finally
    //   128	131	126	finally
  }

  private native int native_getDbLookaside();

  private native void native_key(String paramString)
    throws SQLException;

  private native void native_key(char[] paramArrayOfChar)
    throws SQLException;

  private native void native_rawExecSQL(String paramString);

  private native int native_status(int paramInt, boolean paramBoolean);

  public static SQLiteDatabase openDatabase(String paramString1, String paramString2, CursorFactory paramCursorFactory, int paramInt)
  {
    return openDatabase(paramString1, paramString2, paramCursorFactory, 268435456, null);
  }

  // ERROR //
  public static SQLiteDatabase openDatabase(String paramString1, String paramString2, CursorFactory paramCursorFactory, int paramInt, SQLiteDatabaseHook paramSQLiteDatabaseHook)
  {
    // Byte code:
    //   0: new 2	net/sqlcipher/database/SQLiteDatabase
    //   3: dup
    //   4: aload_0
    //   5: aload_1
    //   6: aload_2
    //   7: iload_3
    //   8: aload 4
    //   10: invokespecial 137	net/sqlcipher/database/SQLiteDatabase:<init>	(Ljava/lang/String;Ljava/lang/String;Lnet/sqlcipher/database/SQLiteDatabase$CursorFactory;ILnet/sqlcipher/database/SQLiteDatabaseHook;)V
    //   13: astore 5
    //   15: getstatic 701	net/sqlcipher/database/SQLiteDebug:DEBUG_SQL_STATEMENTS	Z
    //   18: ifeq +9 -> 27
    //   21: aload 5
    //   23: aload_0
    //   24: invokespecial 703	net/sqlcipher/database/SQLiteDatabase:enableSqlTracing	(Ljava/lang/String;)V
    //   27: getstatic 706	net/sqlcipher/database/SQLiteDebug:DEBUG_SQL_TIME	Z
    //   30: ifeq +9 -> 39
    //   33: aload 5
    //   35: aload_0
    //   36: invokespecial 708	net/sqlcipher/database/SQLiteDatabase:enableSqlProfiling	(Ljava/lang/String;)V
    //   39: invokestatic 451	net/sqlcipher/database/SQLiteDatabase$ActiveDatabases:getInstance	()Lnet/sqlcipher/database/SQLiteDatabase$ActiveDatabases;
    //   42: invokestatic 455	net/sqlcipher/database/SQLiteDatabase$ActiveDatabases:access$000	(Lnet/sqlcipher/database/SQLiteDatabase$ActiveDatabases;)Ljava/util/HashSet;
    //   45: new 460	java/lang/ref/WeakReference
    //   48: dup
    //   49: aload 5
    //   51: invokespecial 711	java/lang/ref/WeakReference:<init>	(Ljava/lang/Object;)V
    //   54: invokevirtual 712	java/util/HashSet:add	(Ljava/lang/Object;)Z
    //   57: pop
    //   58: aload 5
    //   60: areturn
    //   61: astore 6
    //   63: ldc 62
    //   65: new 294	java/lang/StringBuilder
    //   68: dup
    //   69: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   72: ldc_w 714
    //   75: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   78: aload_0
    //   79: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   82: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   85: aload 6
    //   87: invokestatic 265	android/util/Log:e	(Ljava/lang/String;Ljava/lang/String;Ljava/lang/Throwable;)I
    //   90: pop
    //   91: aload_0
    //   92: ldc_w 716
    //   95: invokevirtual 720	java/lang/String:equalsIgnoreCase	(Ljava/lang/String;)Z
    //   98: ifne +15 -> 113
    //   101: new 564	java/io/File
    //   104: dup
    //   105: aload_0
    //   106: invokespecial 642	java/io/File:<init>	(Ljava/lang/String;)V
    //   109: invokevirtual 723	java/io/File:delete	()Z
    //   112: pop
    //   113: new 2	net/sqlcipher/database/SQLiteDatabase
    //   116: dup
    //   117: aload_0
    //   118: aload_1
    //   119: aload_2
    //   120: iload_3
    //   121: aload 4
    //   123: invokespecial 137	net/sqlcipher/database/SQLiteDatabase:<init>	(Ljava/lang/String;Ljava/lang/String;Lnet/sqlcipher/database/SQLiteDatabase$CursorFactory;ILnet/sqlcipher/database/SQLiteDatabaseHook;)V
    //   126: astore 5
    //   128: goto -89 -> 39
    //   131: astore 6
    //   133: goto -70 -> 63
    //
    // Exception table:
    //   from	to	target	type
    //   0	15	61	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   15	27	131	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   27	39	131	net/sqlcipher/database/SQLiteDatabaseCorruptException
  }

  public static SQLiteDatabase openOrCreateDatabase(File paramFile, String paramString, CursorFactory paramCursorFactory)
  {
    return openOrCreateDatabase(paramFile.getPath(), paramString, paramCursorFactory, null);
  }

  public static SQLiteDatabase openOrCreateDatabase(File paramFile, String paramString, CursorFactory paramCursorFactory, SQLiteDatabaseHook paramSQLiteDatabaseHook)
  {
    return openOrCreateDatabase(paramFile.getPath(), paramString, paramCursorFactory, paramSQLiteDatabaseHook);
  }

  public static SQLiteDatabase openOrCreateDatabase(String paramString1, String paramString2, CursorFactory paramCursorFactory)
  {
    return openDatabase(paramString1, paramString2, paramCursorFactory, 268435456, null);
  }

  public static SQLiteDatabase openOrCreateDatabase(String paramString1, String paramString2, CursorFactory paramCursorFactory, SQLiteDatabaseHook paramSQLiteDatabaseHook)
  {
    return openDatabase(paramString1, paramString2, paramCursorFactory, 268435456, paramSQLiteDatabaseHook);
  }

  public static native int releaseMemory();

  public static native void setICURoot(String paramString);

  private void unlockForced()
  {
    if ((SQLiteDebug.DEBUG_LOCK_TIME_TRACKING) && (this.mLock.getHoldCount() == 1))
      checkLockHoldTime();
    this.mLock.unlock();
  }

  public static void upgradeDatabaseFormatFromVersion1To2(File paramFile, String paramString)
    throws Exception
  {
    SQLiteDatabaseHook local1 = new SQLiteDatabaseHook()
    {
      public void postKey(SQLiteDatabase paramAnonymousSQLiteDatabase)
      {
        paramAnonymousSQLiteDatabase.execSQL("PRAGMA cipher_default_use_hmac = on");
      }

      public void preKey(SQLiteDatabase paramAnonymousSQLiteDatabase)
      {
        paramAnonymousSQLiteDatabase.execSQL("PRAGMA cipher_default_use_hmac = off");
      }
    };
    try
    {
      File localFile = File.createTempFile("temp", "db", paramFile.getParentFile());
      SQLiteDatabase localSQLiteDatabase = openOrCreateDatabase(paramFile, paramString, null, local1);
      Object[] arrayOfObject = new Object[1];
      arrayOfObject[0] = localFile.getAbsolutePath();
      localSQLiteDatabase.rawExecSQL(String.format("ATTACH DATABASE '%s' as newdb", arrayOfObject));
      localSQLiteDatabase.rawExecSQL("SELECT sqlcipher_export('newdb')");
      localSQLiteDatabase.rawExecSQL("DETACH DATABASE newdb");
      localSQLiteDatabase.close();
      if (1 != 0)
      {
        paramFile.delete();
        localFile.renameTo(paramFile);
      }
      return;
    }
    catch (Exception localException)
    {
      throw localException;
    }
  }

  private boolean yieldIfContendedHelper(boolean paramBoolean, long paramLong)
  {
    if (this.mLock.getQueueLength() == 0)
    {
      this.mLockAcquiredWallTime = SystemClock.elapsedRealtime();
      this.mLockAcquiredThreadTime = Debug.threadCpuTimeNanos();
      return false;
    }
    setTransactionSuccessful();
    SQLiteTransactionListener localSQLiteTransactionListener = this.mTransactionListener;
    endTransaction();
    if ((paramBoolean) && (isDbLockedByCurrentThread()))
      throw new IllegalStateException("Db locked more than once. yielfIfContended cannot yield");
    long l1;
    if (paramLong > 0L)
      l1 = paramLong;
    while (true)
    {
      long l2;
      if (l1 > 0L)
      {
        if (l1 >= 1000L)
          break label122;
        l2 = l1;
      }
      try
      {
        while (true)
        {
          Thread.sleep(l2);
          l1 -= 1000L;
          if (this.mLock.getQueueLength() != 0)
            break;
          beginTransactionWithListener(localSQLiteTransactionListener);
          return true;
          label122: l2 = 1000L;
        }
      }
      catch (InterruptedException localInterruptedException)
      {
        while (true)
          Thread.interrupted();
      }
    }
  }

  void addSQLiteClosable(SQLiteClosable paramSQLiteClosable)
  {
    lock();
    try
    {
      this.mPrograms.put(paramSQLiteClosable, null);
      return;
    }
    finally
    {
      unlock();
    }
  }

  void addToCompiledQueries(String paramString, SQLiteCompiledSql paramSQLiteCompiledSql)
  {
    if (this.mMaxSqlCacheSize == 0)
    {
      if (SQLiteDebug.DEBUG_SQL_CACHE)
        Log.v("Database", "|NOT adding_sql_to_cache|" + getPath() + "|" + paramString);
      return;
    }
    synchronized (this.mCompiledQueries)
    {
      if ((SQLiteCompiledSql)this.mCompiledQueries.get(paramString) != null)
        return;
    }
    if (this.mCompiledQueries.size() == this.mMaxSqlCacheSize)
    {
      int i = 1 + this.mCacheFullWarnings;
      this.mCacheFullWarnings = i;
      if (i == 1)
        Log.w("Database", "Reached MAX size for compiled-sql statement cache for database " + getPath() + "; i.e., NO space for this sql statement in cache: " + paramString + ". Please change your sql statements to use '?' for " + "bindargs, instead of using actual values");
    }
    while (true)
    {
      return;
      this.mCompiledQueries.put(paramString, paramSQLiteCompiledSql);
      if (SQLiteDebug.DEBUG_SQL_CACHE)
        Log.v("Database", "|adding_sql_to_cache|" + getPath() + "|" + this.mCompiledQueries.size() + "|" + paramString);
    }
  }

  public void beginTransaction()
  {
    beginTransactionWithListener(null);
  }

  public void beginTransactionWithListener(SQLiteTransactionListener paramSQLiteTransactionListener)
  {
    lockForced();
    if (!isOpen())
      throw new IllegalStateException("database not open");
    try
    {
      if (this.mLock.getHoldCount() <= 1)
        break label86;
      if (this.mInnerTransactionIsSuccessful)
      {
        IllegalStateException localIllegalStateException = new IllegalStateException("Cannot call beginTransaction between calling setTransactionSuccessful and endTransaction");
        Log.e("Database", "beginTransaction() failed", localIllegalStateException);
        throw localIllegalStateException;
      }
    }
    finally
    {
      if (0 == 0)
        unlockForced();
    }
    if (1 == 0)
      unlockForced();
    while (true)
    {
      return;
      label86: execSQL("BEGIN EXCLUSIVE;");
      this.mTransactionListener = paramSQLiteTransactionListener;
      this.mTransactionIsSuccessful = true;
      this.mInnerTransactionIsSuccessful = false;
      if (paramSQLiteTransactionListener != null);
      try
      {
        paramSQLiteTransactionListener.onBegin();
        if (1 != 0)
          continue;
        unlockForced();
        return;
      }
      catch (RuntimeException localRuntimeException)
      {
        execSQL("ROLLBACK;");
        throw localRuntimeException;
      }
    }
  }

  public void close()
  {
    if (!isOpen())
      return;
    lock();
    try
    {
      closeClosable();
      onAllReferencesReleased();
      return;
    }
    finally
    {
      unlock();
    }
  }

  public SQLiteStatement compileStatement(String paramString)
    throws SQLException
  {
    lock();
    if (!isOpen())
      throw new IllegalStateException("database not open");
    try
    {
      SQLiteStatement localSQLiteStatement = new SQLiteStatement(this, paramString);
      return localSQLiteStatement;
    }
    finally
    {
      unlock();
    }
  }

  public int delete(String paramString1, String paramString2, String[] paramArrayOfString)
  {
    lock();
    if (!isOpen())
      throw new IllegalStateException("database not open");
    SQLiteStatement localSQLiteStatement = null;
    while (true)
    {
      try
      {
        StringBuilder localStringBuilder = new StringBuilder().append("DELETE FROM ").append(paramString1);
        boolean bool = TextUtils.isEmpty(paramString2);
        localSQLiteStatement = null;
        if (!bool)
        {
          str = " WHERE " + paramString2;
          localSQLiteStatement = compileStatement(str);
          if (paramArrayOfString != null)
          {
            int i = paramArrayOfString.length;
            int j = 0;
            if (j < i)
            {
              DatabaseUtils.bindObjectToProgram(localSQLiteStatement, j + 1, paramArrayOfString[j]);
              j++;
              continue;
            }
          }
          localSQLiteStatement.execute();
          int k = lastChangeCount();
          return k;
        }
      }
      catch (SQLiteDatabaseCorruptException localSQLiteDatabaseCorruptException)
      {
        onCorruption();
        throw localSQLiteDatabaseCorruptException;
      }
      finally
      {
        if (localSQLiteStatement != null)
          localSQLiteStatement.close();
        unlock();
      }
      String str = "";
    }
  }

  public void endTransaction()
  {
    if (!isOpen())
      throw new IllegalStateException("database not open");
    if (!this.mLock.isHeldByCurrentThread())
      throw new IllegalStateException("no transaction pending");
    try
    {
      if (this.mInnerTransactionIsSuccessful)
        this.mInnerTransactionIsSuccessful = false;
      while (true)
      {
        int i = this.mLock.getHoldCount();
        if (i == 1)
          break;
        return;
        this.mTransactionIsSuccessful = false;
      }
    }
    finally
    {
      this.mTransactionListener = null;
      unlockForced();
    }
    SQLiteTransactionListener localSQLiteTransactionListener = this.mTransactionListener;
    Object localObject2 = null;
    if (localSQLiteTransactionListener != null);
    try
    {
      if (this.mTransactionIsSuccessful)
        this.mTransactionListener.onCommit();
      while (this.mTransactionIsSuccessful)
      {
        execSQL("COMMIT;");
        this.mTransactionListener = null;
        unlockForced();
        return;
        this.mTransactionListener.onRollback();
        localObject2 = null;
      }
    }
    catch (RuntimeException localRuntimeException)
    {
      while (true)
      {
        localObject2 = localRuntimeException;
        this.mTransactionIsSuccessful = false;
        continue;
        try
        {
          execSQL("ROLLBACK;");
          if (localObject2 != null)
            throw localObject2;
        }
        catch (SQLException localSQLException)
        {
          Log.d("Database", "exception during rollback, maybe the DB previously performed an auto-rollback");
        }
      }
    }
  }

  public void execSQL(String paramString)
    throws SQLException
  {
    long l = SystemClock.uptimeMillis();
    lock();
    if (!isOpen())
      throw new IllegalStateException("database not open");
    logTimeStat(this.mLastSqlStatement, l, "GETLOCK:");
    try
    {
      native_execSQL(paramString);
      unlock();
      if (paramString == "COMMIT;")
      {
        logTimeStat(this.mLastSqlStatement, l, "COMMIT;");
        return;
      }
    }
    catch (SQLiteDatabaseCorruptException localSQLiteDatabaseCorruptException)
    {
      throw localSQLiteDatabaseCorruptException;
    }
    finally
    {
      unlock();
    }
    logTimeStat(paramString, l, null);
  }

  public void execSQL(String paramString, Object[] paramArrayOfObject)
    throws SQLException
  {
    if (paramArrayOfObject == null)
      throw new IllegalArgumentException("Empty bindArgs");
    long l = SystemClock.uptimeMillis();
    lock();
    if (!isOpen())
      throw new IllegalStateException("database not open");
    SQLiteStatement localSQLiteStatement = null;
    try
    {
      localSQLiteStatement = compileStatement(paramString);
      if (paramArrayOfObject != null)
      {
        int i = paramArrayOfObject.length;
        for (int j = 0; j < i; j++)
          DatabaseUtils.bindObjectToProgram(localSQLiteStatement, j + 1, paramArrayOfObject[j]);
      }
      localSQLiteStatement.execute();
      if (localSQLiteStatement != null)
        localSQLiteStatement.close();
      unlock();
      logTimeStat(paramString, l);
      return;
    }
    catch (SQLiteDatabaseCorruptException localSQLiteDatabaseCorruptException)
    {
      onCorruption();
      throw localSQLiteDatabaseCorruptException;
    }
    finally
    {
      if (localSQLiteStatement != null)
        localSQLiteStatement.close();
      unlock();
    }
  }

  protected void finalize()
  {
    if (isOpen())
    {
      Log.e("Database", "close() was never explicitly called on database '" + this.mPath + "' ", this.mStackTrace);
      closeClosable();
      onAllReferencesReleased();
    }
  }

  SQLiteCompiledSql getCompiledStatementForSql(String paramString)
  {
    while (true)
    {
      synchronized (this.mCompiledQueries)
      {
        if (this.mMaxSqlCacheSize == 0)
        {
          if (SQLiteDebug.DEBUG_SQL_CACHE)
            Log.v("Database", "|cache NOT found|" + getPath());
          return null;
        }
        SQLiteCompiledSql localSQLiteCompiledSql = (SQLiteCompiledSql)this.mCompiledQueries.get(paramString);
        boolean bool;
        if (localSQLiteCompiledSql != null)
        {
          bool = true;
          if (bool)
          {
            this.mNumCacheHits = (1 + this.mNumCacheHits);
            if (SQLiteDebug.DEBUG_SQL_CACHE)
              Log.v("Database", "|cache_stats|" + getPath() + "|" + this.mCompiledQueries.size() + "|" + this.mNumCacheHits + "|" + this.mNumCacheMisses + "|" + bool + "|" + this.mTimeOpened + "|" + this.mTimeClosed + "|" + paramString);
            return localSQLiteCompiledSql;
          }
        }
        else
        {
          bool = false;
        }
      }
      this.mNumCacheMisses = (1 + this.mNumCacheMisses);
    }
  }

  public int getMaxSqlCacheSize()
  {
    try
    {
      int i = this.mMaxSqlCacheSize;
      return i;
    }
    finally
    {
      localObject = finally;
      throw localObject;
    }
  }

  // ERROR //
  public long getMaximumSize()
  {
    // Byte code:
    //   0: aconst_null
    //   1: astore_1
    //   2: aload_0
    //   3: invokevirtual 665	net/sqlcipher/database/SQLiteDatabase:lock	()V
    //   6: aload_0
    //   7: invokevirtual 413	net/sqlcipher/database/SQLiteDatabase:isOpen	()Z
    //   10: ifne +14 -> 24
    //   13: new 405	java/lang/IllegalStateException
    //   16: dup
    //   17: ldc_w 843
    //   20: invokespecial 408	java/lang/IllegalStateException:<init>	(Ljava/lang/String;)V
    //   23: athrow
    //   24: new 531	net/sqlcipher/database/SQLiteStatement
    //   27: dup
    //   28: aload_0
    //   29: ldc_w 942
    //   32: invokespecial 536	net/sqlcipher/database/SQLiteStatement:<init>	(Lnet/sqlcipher/database/SQLiteDatabase;Ljava/lang/String;)V
    //   35: astore_2
    //   36: aload_2
    //   37: invokevirtual 539	net/sqlcipher/database/SQLiteStatement:simpleQueryForLong	()J
    //   40: lstore 4
    //   42: aload_0
    //   43: invokevirtual 501	net/sqlcipher/database/SQLiteDatabase:getPageSize	()J
    //   46: lstore 6
    //   48: lload 6
    //   50: lload 4
    //   52: lmul
    //   53: lstore 8
    //   55: aload_2
    //   56: ifnull +7 -> 63
    //   59: aload_2
    //   60: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   63: aload_0
    //   64: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   67: lload 8
    //   69: lreturn
    //   70: astore_3
    //   71: aload_1
    //   72: ifnull +7 -> 79
    //   75: aload_1
    //   76: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   79: aload_0
    //   80: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   83: aload_3
    //   84: athrow
    //   85: astore_3
    //   86: aload_2
    //   87: astore_1
    //   88: goto -17 -> 71
    //
    // Exception table:
    //   from	to	target	type
    //   24	36	70	finally
    //   36	48	85	finally
  }

  // ERROR //
  public long getPageSize()
  {
    // Byte code:
    //   0: aconst_null
    //   1: astore_1
    //   2: aload_0
    //   3: invokevirtual 665	net/sqlcipher/database/SQLiteDatabase:lock	()V
    //   6: aload_0
    //   7: invokevirtual 413	net/sqlcipher/database/SQLiteDatabase:isOpen	()Z
    //   10: ifne +14 -> 24
    //   13: new 405	java/lang/IllegalStateException
    //   16: dup
    //   17: ldc_w 843
    //   20: invokespecial 408	java/lang/IllegalStateException:<init>	(Ljava/lang/String;)V
    //   23: athrow
    //   24: new 531	net/sqlcipher/database/SQLiteStatement
    //   27: dup
    //   28: aload_0
    //   29: ldc_w 944
    //   32: invokespecial 536	net/sqlcipher/database/SQLiteStatement:<init>	(Lnet/sqlcipher/database/SQLiteDatabase;Ljava/lang/String;)V
    //   35: astore_2
    //   36: aload_2
    //   37: invokevirtual 539	net/sqlcipher/database/SQLiteStatement:simpleQueryForLong	()J
    //   40: lstore 4
    //   42: aload_2
    //   43: ifnull +7 -> 50
    //   46: aload_2
    //   47: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   50: aload_0
    //   51: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   54: lload 4
    //   56: lreturn
    //   57: astore_3
    //   58: aload_1
    //   59: ifnull +7 -> 66
    //   62: aload_1
    //   63: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   66: aload_0
    //   67: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   70: aload_3
    //   71: athrow
    //   72: astore_3
    //   73: aload_2
    //   74: astore_1
    //   75: goto -17 -> 58
    //
    // Exception table:
    //   from	to	target	type
    //   24	36	57	finally
    //   36	42	72	finally
  }

  public final String getPath()
  {
    return this.mPath;
  }

  public Map<String, String> getSyncedTables()
  {
    HashMap localHashMap;
    synchronized (this.mSyncUpdateInfo)
    {
      localHashMap = new HashMap();
      Iterator localIterator = this.mSyncUpdateInfo.keySet().iterator();
      while (localIterator.hasNext())
      {
        String str = (String)localIterator.next();
        SyncUpdateInfo localSyncUpdateInfo = (SyncUpdateInfo)this.mSyncUpdateInfo.get(str);
        if (localSyncUpdateInfo.deletedTable != null)
          localHashMap.put(str, localSyncUpdateInfo.deletedTable);
      }
    }
    return localHashMap;
  }

  // ERROR //
  public int getVersion()
  {
    // Byte code:
    //   0: aconst_null
    //   1: astore_1
    //   2: aload_0
    //   3: invokevirtual 665	net/sqlcipher/database/SQLiteDatabase:lock	()V
    //   6: aload_0
    //   7: invokevirtual 413	net/sqlcipher/database/SQLiteDatabase:isOpen	()Z
    //   10: ifne +14 -> 24
    //   13: new 405	java/lang/IllegalStateException
    //   16: dup
    //   17: ldc_w 843
    //   20: invokespecial 408	java/lang/IllegalStateException:<init>	(Ljava/lang/String;)V
    //   23: athrow
    //   24: new 531	net/sqlcipher/database/SQLiteStatement
    //   27: dup
    //   28: aload_0
    //   29: ldc_w 956
    //   32: invokespecial 536	net/sqlcipher/database/SQLiteStatement:<init>	(Lnet/sqlcipher/database/SQLiteDatabase;Ljava/lang/String;)V
    //   35: astore_2
    //   36: aload_2
    //   37: invokevirtual 539	net/sqlcipher/database/SQLiteStatement:simpleQueryForLong	()J
    //   40: lstore 4
    //   42: lload 4
    //   44: l2i
    //   45: istore 6
    //   47: aload_2
    //   48: ifnull +7 -> 55
    //   51: aload_2
    //   52: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   55: aload_0
    //   56: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   59: iload 6
    //   61: ireturn
    //   62: astore_3
    //   63: aload_1
    //   64: ifnull +7 -> 71
    //   67: aload_1
    //   68: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   71: aload_0
    //   72: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   75: aload_3
    //   76: athrow
    //   77: astore_3
    //   78: aload_2
    //   79: astore_1
    //   80: goto -17 -> 63
    //
    // Exception table:
    //   from	to	target	type
    //   24	36	62	finally
    //   36	42	77	finally
  }

  public boolean inTransaction()
  {
    return this.mLock.getHoldCount() > 0;
  }

  public long insert(String paramString1, String paramString2, ContentValues paramContentValues)
  {
    try
    {
      long l = insertWithOnConflict(paramString1, paramString2, paramContentValues, 0);
      return l;
    }
    catch (SQLException localSQLException)
    {
      Log.e("Database", "Error inserting " + paramContentValues, localSQLException);
    }
    return -1L;
  }

  public long insertOrThrow(String paramString1, String paramString2, ContentValues paramContentValues)
    throws SQLException
  {
    return insertWithOnConflict(paramString1, paramString2, paramContentValues, 0);
  }

  public long insertWithOnConflict(String paramString1, String paramString2, ContentValues paramContentValues, int paramInt)
  {
    if (!isOpen())
      throw new IllegalStateException("database not open");
    StringBuilder localStringBuilder1 = new StringBuilder(152);
    localStringBuilder1.append("INSERT");
    localStringBuilder1.append(CONFLICT_VALUES[paramInt]);
    localStringBuilder1.append(" INTO ");
    localStringBuilder1.append(paramString1);
    StringBuilder localStringBuilder2 = new StringBuilder(40);
    Set localSet;
    if ((paramContentValues != null) && (paramContentValues.size() > 0))
    {
      localSet = paramContentValues.valueSet();
      Iterator localIterator2 = localSet.iterator();
      localStringBuilder1.append('(');
      int k = 0;
      while (localIterator2.hasNext())
      {
        if (k != 0)
        {
          localStringBuilder1.append(", ");
          localStringBuilder2.append(", ");
        }
        k = 1;
        localStringBuilder1.append((String)((Map.Entry)localIterator2.next()).getKey());
        localStringBuilder2.append('?');
      }
      localStringBuilder1.append(')');
    }
    while (true)
    {
      localStringBuilder1.append(" VALUES(");
      localStringBuilder1.append(localStringBuilder2);
      localStringBuilder1.append(");");
      lock();
      SQLiteStatement localSQLiteStatement = null;
      try
      {
        localSQLiteStatement = compileStatement(localStringBuilder1.toString());
        if (localSet != null)
        {
          int i = localSet.size();
          Iterator localIterator1 = localSet.iterator();
          int j = 0;
          while (true)
            if (j < i)
            {
              Map.Entry localEntry = (Map.Entry)localIterator1.next();
              DatabaseUtils.bindObjectToProgram(localSQLiteStatement, j + 1, localEntry.getValue());
              j++;
              continue;
              localStringBuilder1.append("(" + paramString2 + ") ");
              localStringBuilder2.append("NULL");
              localSet = null;
              break;
            }
        }
        localSQLiteStatement.execute();
        long l = lastInsertRow();
        if (l == -1L)
          Log.e("Database", "Error inserting " + paramContentValues + " using " + localStringBuilder1);
        while (true)
        {
          return l;
          if (Log.isLoggable("Database", 2))
            Log.v("Database", "Inserting row " + l + " from " + paramContentValues + " using " + localStringBuilder1);
        }
      }
      catch (SQLiteDatabaseCorruptException localSQLiteDatabaseCorruptException)
      {
        onCorruption();
        throw localSQLiteDatabaseCorruptException;
      }
      finally
      {
        if (localSQLiteStatement != null)
          localSQLiteStatement.close();
        unlock();
      }
    }
  }

  public boolean isDbLockedByCurrentThread()
  {
    return this.mLock.isHeldByCurrentThread();
  }

  public boolean isDbLockedByOtherThreads()
  {
    return (!this.mLock.isHeldByCurrentThread()) && (this.mLock.isLocked());
  }

  public boolean isInCompiledSqlCache(String paramString)
  {
    synchronized (this.mCompiledQueries)
    {
      boolean bool = this.mCompiledQueries.containsKey(paramString);
      return bool;
    }
  }

  public boolean isOpen()
  {
    return this.mNativeHandle != 0;
  }

  public boolean isReadOnly()
  {
    return (0x1 & this.mFlags) == 1;
  }

  native int lastChangeCount();

  native long lastInsertRow();

  void lock()
  {
    if (!this.mLockingEnabled);
    do
    {
      return;
      this.mLock.lock();
    }
    while ((!SQLiteDebug.DEBUG_LOCK_TIME_TRACKING) || (this.mLock.getHoldCount() != 1));
    this.mLockAcquiredWallTime = SystemClock.elapsedRealtime();
    this.mLockAcquiredThreadTime = Debug.threadCpuTimeNanos();
  }

  void logTimeStat(String paramString, long paramLong)
  {
    logTimeStat(paramString, paramLong, null);
  }

  void logTimeStat(String paramString1, long paramLong, String paramString2)
  {
    this.mLastSqlStatement = paramString1;
    long l = SystemClock.uptimeMillis() - paramLong;
    if ((l == 0L) && (paramString2 == "GETLOCK:"))
      return;
    if (sQueryLogTimeInMillis == 0)
      sQueryLogTimeInMillis = 500;
    if (l >= sQueryLogTimeInMillis);
    int i;
    do
    {
      if (paramString2 != null)
        paramString1 = paramString2 + paramString1;
      if (paramString1.length() > 64)
        paramString1.substring(0, 64);
      if ("unknown" != null)
        break;
      return;
      i = 1 + (int)(100L * l / sQueryLogTimeInMillis);
    }
    while (this.mRandom.nextInt(100) < i);
  }

  public void markTableSyncable(String paramString1, String paramString2)
  {
    markTableSyncable(paramString1, "_id", paramString1, paramString2);
  }

  public void markTableSyncable(String paramString1, String paramString2, String paramString3)
  {
    markTableSyncable(paramString1, paramString2, paramString3, null);
  }

  native void native_execSQL(String paramString)
    throws SQLException;

  native void native_setLocale(String paramString, int paramInt);

  public boolean needUpgrade(int paramInt)
  {
    return paramInt > getVersion();
  }

  protected void onAllReferencesReleased()
  {
    if (isOpen())
    {
      if (SQLiteDebug.DEBUG_SQL_CACHE)
        this.mTimeClosed = getTime();
      dbclose();
    }
  }

  void onCorruption()
  {
    Log.e("Database", "Removing corrupt database: " + this.mPath);
    try
    {
      close();
      return;
    }
    finally
    {
      if (!this.mPath.equalsIgnoreCase(":memory"))
        new File(this.mPath).delete();
    }
  }

  public void purgeFromCompiledSqlCache(String paramString)
  {
    synchronized (this.mCompiledQueries)
    {
      this.mCompiledQueries.remove(paramString);
      return;
    }
  }

  public Cursor query(String paramString1, String[] paramArrayOfString1, String paramString2, String[] paramArrayOfString2, String paramString3, String paramString4, String paramString5)
  {
    return query(false, paramString1, paramArrayOfString1, paramString2, paramArrayOfString2, paramString3, paramString4, paramString5, null);
  }

  public Cursor query(String paramString1, String[] paramArrayOfString1, String paramString2, String[] paramArrayOfString2, String paramString3, String paramString4, String paramString5, String paramString6)
  {
    return query(false, paramString1, paramArrayOfString1, paramString2, paramArrayOfString2, paramString3, paramString4, paramString5, paramString6);
  }

  public Cursor query(boolean paramBoolean, String paramString1, String[] paramArrayOfString1, String paramString2, String[] paramArrayOfString2, String paramString3, String paramString4, String paramString5, String paramString6)
  {
    return queryWithFactory(null, paramBoolean, paramString1, paramArrayOfString1, paramString2, paramArrayOfString2, paramString3, paramString4, paramString5, paramString6);
  }

  public Cursor queryWithFactory(CursorFactory paramCursorFactory, boolean paramBoolean, String paramString1, String[] paramArrayOfString1, String paramString2, String[] paramArrayOfString2, String paramString3, String paramString4, String paramString5, String paramString6)
  {
    if (!isOpen())
      throw new IllegalStateException("database not open");
    return rawQueryWithFactory(paramCursorFactory, SQLiteQueryBuilder.buildQueryString(paramBoolean, paramString1, paramArrayOfString1, paramString2, paramString3, paramString4, paramString5, paramString6), paramArrayOfString2, findEditTable(paramString1));
  }

  public void rawExecSQL(String paramString)
  {
    long l = SystemClock.uptimeMillis();
    lock();
    if (!isOpen())
      throw new IllegalStateException("database not open");
    logTimeStat(this.mLastSqlStatement, l, "GETLOCK:");
    try
    {
      native_rawExecSQL(paramString);
      unlock();
      if (paramString == "COMMIT;")
      {
        logTimeStat(this.mLastSqlStatement, l, "COMMIT;");
        return;
      }
    }
    catch (SQLiteDatabaseCorruptException localSQLiteDatabaseCorruptException)
    {
      throw localSQLiteDatabaseCorruptException;
    }
    finally
    {
      unlock();
    }
    logTimeStat(paramString, l, null);
  }

  public Cursor rawQuery(String paramString, String[] paramArrayOfString)
  {
    return rawQueryWithFactory(null, paramString, paramArrayOfString, null);
  }

  public Cursor rawQuery(String paramString, String[] paramArrayOfString, int paramInt1, int paramInt2)
  {
    SQLiteCursor localSQLiteCursor = (SQLiteCursor)rawQueryWithFactory(null, paramString, paramArrayOfString, null);
    localSQLiteCursor.setLoadStyle(paramInt1, paramInt2);
    return localSQLiteCursor;
  }

  public Cursor rawQueryWithFactory(CursorFactory paramCursorFactory, String paramString1, String[] paramArrayOfString, String paramString2)
  {
    if (!isOpen())
      throw new IllegalStateException("database not open");
    long l1 = 0L;
    if (this.mSlowQueryThreshold != -1)
      l1 = System.currentTimeMillis();
    SQLiteDirectCursorDriver localSQLiteDirectCursorDriver = new SQLiteDirectCursorDriver(this, paramString1, paramString2);
    if (paramCursorFactory != null);
    try
    {
      while (true)
      {
        Cursor localCursor = localSQLiteDirectCursorDriver.query(paramCursorFactory, paramArrayOfString);
        if (this.mSlowQueryThreshold != -1)
        {
          int j = -1;
          if (localCursor != null)
            j = localCursor.getCount();
          long l3 = System.currentTimeMillis() - l1;
          if (l3 >= this.mSlowQueryThreshold)
          {
            StringBuilder localStringBuilder2 = new StringBuilder().append("query (").append(l3).append(" ms): ").append(localSQLiteDirectCursorDriver.toString()).append(", args are ");
            if (paramArrayOfString == null)
              break;
            str2 = TextUtils.join(",", paramArrayOfString);
            Log.v("Database", str2 + ", count is " + j);
          }
        }
        return new CrossProcessCursorWrapper(localCursor);
        paramCursorFactory = this.mFactory;
      }
      str1 = "<null>";
      Log.v("Database", str1 + ", count is " + i);
      Object localObject1;
      throw localObject1;
    }
    finally
    {
      while (true)
      {
        String str2;
        String str1;
        StringBuilder localStringBuilder1;
        int i;
        if (this.mSlowQueryThreshold != -1)
        {
          i = -1;
          if (0 != 0)
            i = null.getCount();
          long l2 = System.currentTimeMillis() - l1;
          if (l2 >= this.mSlowQueryThreshold)
          {
            localStringBuilder1 = new StringBuilder().append("query (").append(l2).append(" ms): ").append(localSQLiteDirectCursorDriver.toString()).append(", args are ");
            if (paramArrayOfString != null)
            {
              str1 = TextUtils.join(",", paramArrayOfString);
              continue;
              str2 = "<null>";
            }
          }
        }
      }
    }
  }

  void removeSQLiteClosable(SQLiteClosable paramSQLiteClosable)
  {
    lock();
    try
    {
      this.mPrograms.remove(paramSQLiteClosable);
      return;
    }
    finally
    {
      unlock();
    }
  }

  public long replace(String paramString1, String paramString2, ContentValues paramContentValues)
  {
    try
    {
      long l = insertWithOnConflict(paramString1, paramString2, paramContentValues, 5);
      return l;
    }
    catch (SQLException localSQLException)
    {
      Log.e("Database", "Error inserting " + paramContentValues, localSQLException);
    }
    return -1L;
  }

  public long replaceOrThrow(String paramString1, String paramString2, ContentValues paramContentValues)
    throws SQLException
  {
    return insertWithOnConflict(paramString1, paramString2, paramContentValues, 5);
  }

  public void resetCompiledSqlCache()
  {
    synchronized (this.mCompiledQueries)
    {
      this.mCompiledQueries.clear();
      return;
    }
  }

  void rowUpdated(String paramString, long paramLong)
  {
    synchronized (this.mSyncUpdateInfo)
    {
      SyncUpdateInfo localSyncUpdateInfo = (SyncUpdateInfo)this.mSyncUpdateInfo.get(paramString);
      if (localSyncUpdateInfo != null)
        execSQL("UPDATE " + localSyncUpdateInfo.masterTable + " SET _sync_dirty=1 WHERE _id=(SELECT " + localSyncUpdateInfo.foreignKey + " FROM " + paramString + " WHERE _id=" + paramLong + ")");
      return;
    }
  }

  public void setLocale(Locale paramLocale)
  {
    lock();
    try
    {
      native_setLocale(paramLocale.toString(), this.mFlags);
      return;
    }
    finally
    {
      unlock();
    }
  }

  public void setLockingEnabled(boolean paramBoolean)
  {
    this.mLockingEnabled = paramBoolean;
  }

  public void setMaxSqlCacheSize(int paramInt)
  {
    if ((paramInt > 250) || (paramInt < 0))
      try
      {
        throw new IllegalStateException("expected value between 0 and 250");
      }
      finally
      {
      }
    if (paramInt < this.mMaxSqlCacheSize)
      throw new IllegalStateException("cannot set cacheSize to a value less than the value set with previous setMaxSqlCacheSize() call.");
    this.mMaxSqlCacheSize = paramInt;
  }

  // ERROR //
  public long setMaximumSize(long paramLong)
  {
    // Byte code:
    //   0: aconst_null
    //   1: astore_3
    //   2: aload_0
    //   3: invokevirtual 665	net/sqlcipher/database/SQLiteDatabase:lock	()V
    //   6: aload_0
    //   7: invokevirtual 413	net/sqlcipher/database/SQLiteDatabase:isOpen	()Z
    //   10: ifne +14 -> 24
    //   13: new 405	java/lang/IllegalStateException
    //   16: dup
    //   17: ldc_w 843
    //   20: invokespecial 408	java/lang/IllegalStateException:<init>	(Ljava/lang/String;)V
    //   23: athrow
    //   24: aload_0
    //   25: invokevirtual 501	net/sqlcipher/database/SQLiteDatabase:getPageSize	()J
    //   28: lstore 5
    //   30: lload_1
    //   31: lload 5
    //   33: ldiv
    //   34: lstore 7
    //   36: lload_1
    //   37: lload 5
    //   39: lrem
    //   40: lconst_0
    //   41: lcmp
    //   42: ifeq +9 -> 51
    //   45: lload 7
    //   47: lconst_1
    //   48: ladd
    //   49: lstore 7
    //   51: new 531	net/sqlcipher/database/SQLiteStatement
    //   54: dup
    //   55: aload_0
    //   56: new 294	java/lang/StringBuilder
    //   59: dup
    //   60: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   63: ldc_w 1148
    //   66: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   69: lload 7
    //   71: invokevirtual 306	java/lang/StringBuilder:append	(J)Ljava/lang/StringBuilder;
    //   74: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   77: invokespecial 536	net/sqlcipher/database/SQLiteStatement:<init>	(Lnet/sqlcipher/database/SQLiteDatabase;Ljava/lang/String;)V
    //   80: astore 9
    //   82: aload 9
    //   84: invokevirtual 539	net/sqlcipher/database/SQLiteStatement:simpleQueryForLong	()J
    //   87: lstore 10
    //   89: lload 10
    //   91: lload 5
    //   93: lmul
    //   94: lstore 12
    //   96: aload 9
    //   98: ifnull +8 -> 106
    //   101: aload 9
    //   103: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   106: aload_0
    //   107: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   110: lload 12
    //   112: lreturn
    //   113: astore 4
    //   115: aload_3
    //   116: ifnull +7 -> 123
    //   119: aload_3
    //   120: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   123: aload_0
    //   124: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   127: aload 4
    //   129: athrow
    //   130: astore 4
    //   132: aload 9
    //   134: astore_3
    //   135: goto -20 -> 115
    //
    // Exception table:
    //   from	to	target	type
    //   24	36	113	finally
    //   51	82	113	finally
    //   82	89	130	finally
  }

  public void setPageSize(long paramLong)
  {
    execSQL("PRAGMA page_size = " + paramLong);
  }

  public void setTransactionSuccessful()
  {
    if (!isOpen())
      throw new IllegalStateException("database not open");
    if (!this.mLock.isHeldByCurrentThread())
      throw new IllegalStateException("no transaction pending");
    if (this.mInnerTransactionIsSuccessful)
      throw new IllegalStateException("setTransactionSuccessful may only be called once per call to beginTransaction");
    this.mInnerTransactionIsSuccessful = true;
  }

  public void setVersion(int paramInt)
  {
    execSQL("PRAGMA user_version = " + paramInt);
  }

  public int status(int paramInt, boolean paramBoolean)
  {
    return native_status(paramInt, paramBoolean);
  }

  void unlock()
  {
    if (!this.mLockingEnabled)
      return;
    if ((SQLiteDebug.DEBUG_LOCK_TIME_TRACKING) && (this.mLock.getHoldCount() == 1))
      checkLockHoldTime();
    this.mLock.unlock();
  }

  public int update(String paramString1, ContentValues paramContentValues, String paramString2, String[] paramArrayOfString)
  {
    return updateWithOnConflict(paramString1, paramContentValues, paramString2, paramArrayOfString, 0);
  }

  // ERROR //
  public int updateWithOnConflict(String paramString1, ContentValues paramContentValues, String paramString2, String[] paramArrayOfString, int paramInt)
  {
    // Byte code:
    //   0: aload_2
    //   1: ifnull +10 -> 11
    //   4: aload_2
    //   5: invokevirtual 981	android/content/ContentValues:size	()I
    //   8: ifne +14 -> 22
    //   11: new 192	java/lang/IllegalArgumentException
    //   14: dup
    //   15: ldc_w 1167
    //   18: invokespecial 197	java/lang/IllegalArgumentException:<init>	(Ljava/lang/String;)V
    //   21: athrow
    //   22: new 294	java/lang/StringBuilder
    //   25: dup
    //   26: bipush 120
    //   28: invokespecial 974	java/lang/StringBuilder:<init>	(I)V
    //   31: astore 6
    //   33: aload 6
    //   35: ldc_w 1123
    //   38: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   41: pop
    //   42: aload 6
    //   44: getstatic 120	net/sqlcipher/database/SQLiteDatabase:CONFLICT_VALUES	[Ljava/lang/String;
    //   47: iload 5
    //   49: aaload
    //   50: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   53: pop
    //   54: aload 6
    //   56: aload_1
    //   57: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   60: pop
    //   61: aload 6
    //   63: ldc_w 1169
    //   66: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   69: pop
    //   70: aload_2
    //   71: invokevirtual 984	android/content/ContentValues:valueSet	()Ljava/util/Set;
    //   74: astore 11
    //   76: aload 11
    //   78: invokeinterface 342 1 0
    //   83: astore 12
    //   85: aload 12
    //   87: invokeinterface 348 1 0
    //   92: ifeq +58 -> 150
    //   95: aload 6
    //   97: aload 12
    //   99: invokeinterface 352 1 0
    //   104: checkcast 354	java/util/Map$Entry
    //   107: invokeinterface 357 1 0
    //   112: checkcast 106	java/lang/String
    //   115: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   118: pop
    //   119: aload 6
    //   121: ldc_w 1171
    //   124: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   127: pop
    //   128: aload 12
    //   130: invokeinterface 348 1 0
    //   135: ifeq -50 -> 85
    //   138: aload 6
    //   140: ldc_w 989
    //   143: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   146: pop
    //   147: goto -62 -> 85
    //   150: aload_3
    //   151: invokestatic 395	android/text/TextUtils:isEmpty	(Ljava/lang/CharSequence;)Z
    //   154: ifne +19 -> 173
    //   157: aload 6
    //   159: ldc_w 877
    //   162: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   165: pop
    //   166: aload 6
    //   168: aload_3
    //   169: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   172: pop
    //   173: aload_0
    //   174: invokevirtual 665	net/sqlcipher/database/SQLiteDatabase:lock	()V
    //   177: aload_0
    //   178: invokevirtual 413	net/sqlcipher/database/SQLiteDatabase:isOpen	()Z
    //   181: ifne +14 -> 195
    //   184: new 405	java/lang/IllegalStateException
    //   187: dup
    //   188: ldc_w 843
    //   191: invokespecial 408	java/lang/IllegalStateException:<init>	(Ljava/lang/String;)V
    //   194: athrow
    //   195: aconst_null
    //   196: astore 13
    //   198: aload_0
    //   199: aload 6
    //   201: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   204: invokevirtual 879	net/sqlcipher/database/SQLiteDatabase:compileStatement	(Ljava/lang/String;)Lnet/sqlcipher/database/SQLiteStatement;
    //   207: astore 13
    //   209: aload 11
    //   211: invokeinterface 997 1 0
    //   216: istore 18
    //   218: aload 11
    //   220: invokeinterface 342 1 0
    //   225: astore 19
    //   227: iconst_1
    //   228: istore 20
    //   230: iconst_0
    //   231: istore 21
    //   233: iload 21
    //   235: iload 18
    //   237: if_icmpge +34 -> 271
    //   240: aload 13
    //   242: iload 20
    //   244: aload 19
    //   246: invokeinterface 352 1 0
    //   251: checkcast 354	java/util/Map$Entry
    //   254: invokeinterface 1000 1 0
    //   259: invokestatic 885	net/sqlcipher/DatabaseUtils:bindObjectToProgram	(Lnet/sqlcipher/database/SQLiteProgram;ILjava/lang/Object;)V
    //   262: iinc 20 1
    //   265: iinc 21 1
    //   268: goto -35 -> 233
    //   271: aload 4
    //   273: ifnull +39 -> 312
    //   276: aload 4
    //   278: arraylength
    //   279: istore 22
    //   281: iconst_0
    //   282: istore 23
    //   284: iload 23
    //   286: iload 22
    //   288: if_icmpge +24 -> 312
    //   291: aload 13
    //   293: iload 20
    //   295: aload 4
    //   297: iload 23
    //   299: aaload
    //   300: invokevirtual 1175	net/sqlcipher/database/SQLiteStatement:bindString	(ILjava/lang/String;)V
    //   303: iinc 20 1
    //   306: iinc 23 1
    //   309: goto -25 -> 284
    //   312: aload 13
    //   314: invokevirtual 888	net/sqlcipher/database/SQLiteStatement:execute	()V
    //   317: aload_0
    //   318: invokevirtual 891	net/sqlcipher/database/SQLiteDatabase:lastChangeCount	()I
    //   321: istore 24
    //   323: ldc 62
    //   325: iconst_2
    //   326: invokestatic 281	android/util/Log:isLoggable	(Ljava/lang/String;I)Z
    //   329: ifeq +51 -> 380
    //   332: ldc 62
    //   334: new 294	java/lang/StringBuilder
    //   337: dup
    //   338: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   341: ldc_w 1177
    //   344: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   347: iload 24
    //   349: invokevirtual 311	java/lang/StringBuilder:append	(I)Ljava/lang/StringBuilder;
    //   352: ldc_w 1011
    //   355: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   358: aload_2
    //   359: invokevirtual 968	java/lang/StringBuilder:append	(Ljava/lang/Object;)Ljava/lang/StringBuilder;
    //   362: ldc_w 1179
    //   365: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   368: aload 6
    //   370: invokevirtual 968	java/lang/StringBuilder:append	(Ljava/lang/Object;)Ljava/lang/StringBuilder;
    //   373: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   376: invokestatic 819	android/util/Log:v	(Ljava/lang/String;Ljava/lang/String;)I
    //   379: pop
    //   380: aload 13
    //   382: ifnull +8 -> 390
    //   385: aload 13
    //   387: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   390: aload_0
    //   391: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   394: iload 24
    //   396: ireturn
    //   397: astore 17
    //   399: aload_0
    //   400: invokevirtual 894	net/sqlcipher/database/SQLiteDatabase:onCorruption	()V
    //   403: aload 17
    //   405: athrow
    //   406: astore 16
    //   408: aload 13
    //   410: ifnull +8 -> 418
    //   413: aload 13
    //   415: invokevirtual 540	net/sqlcipher/database/SQLiteStatement:close	()V
    //   418: aload_0
    //   419: invokevirtual 679	net/sqlcipher/database/SQLiteDatabase:unlock	()V
    //   422: aload 16
    //   424: athrow
    //   425: astore 14
    //   427: ldc 62
    //   429: new 294	java/lang/StringBuilder
    //   432: dup
    //   433: invokespecial 295	java/lang/StringBuilder:<init>	()V
    //   436: ldc_w 1181
    //   439: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   442: aload_2
    //   443: invokevirtual 968	java/lang/StringBuilder:append	(Ljava/lang/Object;)Ljava/lang/StringBuilder;
    //   446: ldc_w 1011
    //   449: invokevirtual 301	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   452: aload 6
    //   454: invokevirtual 968	java/lang/StringBuilder:append	(Ljava/lang/Object;)Ljava/lang/StringBuilder;
    //   457: invokevirtual 316	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   460: invokestatic 1013	android/util/Log:e	(Ljava/lang/String;Ljava/lang/String;)I
    //   463: pop
    //   464: aload 14
    //   466: athrow
    //
    // Exception table:
    //   from	to	target	type
    //   198	227	397	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   240	262	397	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   276	281	397	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   291	303	397	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   312	380	397	net/sqlcipher/database/SQLiteDatabaseCorruptException
    //   198	227	406	finally
    //   240	262	406	finally
    //   276	281	406	finally
    //   291	303	406	finally
    //   312	380	406	finally
    //   399	406	406	finally
    //   427	467	406	finally
    //   198	227	425	net/sqlcipher/SQLException
    //   240	262	425	net/sqlcipher/SQLException
    //   276	281	425	net/sqlcipher/SQLException
    //   291	303	425	net/sqlcipher/SQLException
    //   312	380	425	net/sqlcipher/SQLException
  }

  @Deprecated
  public boolean yieldIfContended()
  {
    return yieldIfContendedHelper(false, -1L);
  }

  public boolean yieldIfContendedSafely()
  {
    return yieldIfContendedHelper(true, -1L);
  }

  public boolean yieldIfContendedSafely(long paramLong)
  {
    return yieldIfContendedHelper(true, paramLong);
  }

  static class ActiveDatabases
  {
    private static final ActiveDatabases activeDatabases = new ActiveDatabases();
    private HashSet<WeakReference<SQLiteDatabase>> mActiveDatabases = new HashSet();

    static ActiveDatabases getInstance()
    {
      return activeDatabases;
    }
  }

  public static abstract interface CursorFactory
  {
    public abstract Cursor newCursor(SQLiteDatabase paramSQLiteDatabase, SQLiteCursorDriver paramSQLiteCursorDriver, String paramString, SQLiteQuery paramSQLiteQuery);
  }

  private static class SyncUpdateInfo
  {
    String deletedTable;
    String foreignKey;
    String masterTable;

    SyncUpdateInfo(String paramString1, String paramString2, String paramString3)
    {
      this.masterTable = paramString1;
      this.deletedTable = paramString2;
      this.foreignKey = paramString3;
    }
  }
}

/* Location:           /tmp/apk/wickr/classes_dex2jar.jar
 * Qualified Name:     net.sqlcipher.database.SQLiteDatabase
 * JD-Core Version:    0.6.2
 */